# Installation guide

#### 1. Install required packages
  If using spfx version <= 1.7
  ```bash
  npm i -DE http://libs.it.ntnu.no/npm/tslint-config-ntnu-0.4.2.tgz tslint-config-prettier tslint-react typescript-tslint-plugin prettier
  ```

  else use

  ```bash
  npm i -DE http://libs.it.ntnu.no/npm/tslint-config-ntnu-0.5.0.tgz tslint-config-prettier tslint-react typescript-tslint-plugin prettier
  ```

#### 2. __Replace__ content in  `tslint.json` with.

  ``` json
    {
    "extends": ["tslint-config-ntnu", "tslint-react", "tslint-config-prettier"]
    }
  ```


#### 3. Add `typescript-tslint-plugin` in `tsconfig.json`

  ``` json
    {
      ...
      "compilerOptions": {
        ...
        "plugins": [
          {
            "name": "typescript-tslint-plugin"
          }
        ],
      }
    }
  ```
  




#### 4. On each project a `.prettierrc` file must exsist on same level as `package.json`, with following config

  ``` json
    {
    "printWidth": 100,
    "useTabs": false,
    "tabWidth": 2,
    "semi": true,
    "singleQuote": true,
    "trailingComma": "all",
    "bracketSpacing": true,
    "jsxBracketSameLine": false,
    "arrowParens": "always",
    "proseWrap": "never",
    "endOfLine": "lf"
    }
  ```
#### 5. Add commands in `package.json`

  ```json
  {
    ...,
    "scripts": {
      ...
      "lint": "tslint --project tsconfig.json --config tslint.json",
      "format": "prettier --write src/**/*.{ts,tsx}"
    }
  }
  ```

#### 6. If you don't have it already installed, Prettier [available on VSCode](https://marketplace.visualstudio.com/items?itemName=esbenp.prettier-vscode) 

- __If__ on Visual Studios Code, and you want 

#### 7. If you're using VSCode, make sure that you are using correct the projects TS version 


# React

  <a name="state"></a><a name="1.1"></a>
  - #### [1.1](#state) Never set state more than once per function.

    Why ?
    > setState is async as has no given guarantee when it's updated, which can lead to quirky bugs, and or preformence loss caused by multiple rerenders.

    ```javascript
    this.setState({a : 1})
    this.setState({b : 3})

    console.log(this.state.a, this.state.b) //It's a cointoss on who actually fires
    ```

  <a name="mutating-props"></a><a name="1.2"></a>
  - #### [1.2](#mutating-props) Never mutate props, props are read-only.

    > Why ? All props are assumed immutable, and will lead to propagating bugs, and are usually hard to source. 
    > And React relies on object id to detect when to rerender, causing changes not to update.

    ```html
    // NOTE: inline lambda is used here for abbreviation of context, do not use in wild
    // In TipComponent.jsx / .tsx
      return (
        <div>
          <button
            label="Legg til en krone"
            onClick={() => this.props.value = this.props.value + 1}
          />
          <span>TIP is: {this.props.value}kr</span> 
        </div>)
        // Won't rerender value and will be stuck at initial value.


      // ShoppingCart.jsx / .tsx
      return (
        <div>
          <TipComponent
            value={this.state.value}
          />
          <button
            label="Legg til en krone"
            onClick={() => tip(this.state.value)} 
            // This function will send actuall number
          />
        </div>)
    ```

  <a name="file-structure"></a><a name="1.3"></a>
  - #### [1.3](#file-structure) Use same file structure so it's easier to navigate.
  
    Why? 
    > Having a commone file-structure, makes it easier to navigate for all parties involved  

    
    <a name="file-structure-class"></a><a name="1.3.1"></a>
     #### [1.3.1](#file-structure-class) For a class component


    ```tsx
    
      // Import third-party libaries on top, separate project improts with newline.
      import * as React from 'react'
      import { connect } from 'react-redux';

      import { CustomComponent } from '../myProject/CustomComponent'

      // Interface declaration (props/state) (less than 50 lines,
      // if more concider moving out in own file)
      export interface ExampleComponentProps {...}
      export interface ExampleComponentState {...}


      class ExampleComponent extends React.Component<
        ExampleComponentProps, 
        ExampleComponentState
      > {
        // Private variables.
        private _imAPrivateVariable: string = "I'm a private variable"

        // constructor / lifecycle functions
        consturctor()
        public componentDidMount = () => {...}
        public componentDidUpdate = () => {...}
        public componentWillUnmount = () => {...}

        // Custom functions, all should be private, alphabetically ordered
        private  myCoolFunction = () => {...}

        
        //Render method should be last in scope
        public render(): React.ReactElement<ExampleComponentProps> {
          // Variables/functions that need ot be calculated inside render
          const variablesThatNeedToBeCalculatedOnEachRender = 3 + 4 

          // return statement last
          return(
            ...
            )
        }
      }

      // If using Redux, this is where we map state and dispatch and  our props
      const mapStateToProps = (state) => ({...})
      const mapDispatchToProps = (dispatch) => ({...})

      // If using a HOC this is where we apply it 
      export default connect(
        mapStateToProps,
        mapDispatchToProps
      )(ExampleComponent);
   
    ```

    <a name="file-structure-function"></a><a name="1.3.2"></a>
     #### [1.3.2](#file-structure-class) For functional components (Preffered way from `React:16.8.0`)

    ```tsx
      // If version of TS is wonky and doens't allow this syntax, following is allowed      
      // import * as React from 'react'
      // const  {useState, useEffect, ...} = React

       // Import third-party libaries on top, separate project imports with newline.
      import * as React, {useState, useEffect, ...} from 'react'
      import { connect } from 'react-redux';

      import { CustomComponent } from '../myProject/CustomComponent'

      // Interface declaration (props/state) (less than 50 lines,
      // if more concider moving out in own file)
      export interface ExampleComponentProps {...}

      const ExampleComponent = ({destructed, props}: ExampleComponentProps) => {
        // Hooks are defined at the top, state, effect, memos ect....
        const [value, setValue] = useState<number>(0);
        
        useEffect(() => {
          ...
        }, [...dependencies]);

        // custom functions

        const onChange = () => {...};

        // return render
        return (
          <div>
            ...
          </div>
        );
      }
    ```

# General

<a name="testing-for-falsy-truthy"></a><a name="2.1"></a>
- #### [2.1](#testing-for-falsy-truthy) When testing for truthy statements use !! prefix

  Why? 
  > Easier to understand to understand intention, and helps when doint comparisons

  ```javascript
    const a = []
    const b = {}
      
    // This is fine if type-checking, but dat for truthy statement
    console.log(a === b) // false
    
    // This is for truthy statements
    console.log(!!a === !!b) // true
  ```


<a name="console-commands"></a><a name="2.2"></a>
- #### [2.2](#console-commands) Console functions should never appear in production.

  Why?
  > Creates a lot of noise of user and, may help malicious actors to unscramble vulnerable parts of our code. No. 1 rule is to ban console statements when pushing into git, no client side code should log anything to console.
  
  If any console statements are to be in client side solution level should be `error` or more severe


<a name="avoid-deprecated"></a><a name="2.3"></a>
- #### [2.3](#avoid-deprecated) Avoid deprecated functions and patterns.

   Why?  
  > Patterns/functions are deprecated either because they cause vulnerabillities or cause alot of headache for developers.
  > Using something deprecated is giving yourself/others more work in the future that usually don't have the same understading of your code base as you do now. 

<a name="think-immutabillity"></a><a name="2.4"></a>
- #### [2.4](#think-immutabillity) Think immutable!

  > We are not enforcing immutabillity for now, but reccoment to develop thinking immutabillity.

  
  #### What you can do to start thinking immutable?

  Avoid `let` declarations, unsing const enforces you to start thinkinkin how to not mutate an object.
  ```javascript
  let x = 5
  x = x + 3 // This is "fine" but allows for sideeffects like this.

  const y = 5
  y = y + 3 // SyntaxError, as this is a sideeffect and y can't be declared againg,
  y++       // SyntaxError
  ```

  __NOTE__: Javascript doesn't enforce non mutations of properties of const objects, but that doesn't mean that we should.
  ```javascript
  const obj = {a: 1, b:2};
  
  const setObjProperty = (property, value) => {
    return obj[property] = value
  };
  // This will create a side effect on obj.
  const updatedObj = setObjProperty('a', 4)
  console.log(updatedObj); //{a: 4, b: 2}
  console.log(obj); //{a: 4, b: 2}


  // Recomended is to use the `...` spread opperator
  const obj = {a: 1, b:2};
  
  const setObjProperty = (property, value) => {
    return {...obj, [property]: value}
  };
  // This will create a side effect on obj.
  const updatedObj = setObjProperty('a', 4)
  console.log(updatedObj); //{a: 4, b: 2}
  console.log(obj); //{a: 1, b: 2}
  ```

  Try using JS's immutable functions like `.map` `.filter` `.reduce` 
  ```javascript
  const ImmutableList = [1,2,3,4] 
  const powerOfTwoWithSideEffects = list => {
    // This works, but causes ambiguity and mutates original list.
    list.forEach((value, index) => {
      list[index] = value ** 2
    })
    return list
  }
  console.log(powerOfTwoWithSideEffects(ImmutableList)) //[1, 4, 9, 16]
  console.log(ImmutableList) //[1, 4, 9, 16]

  // Could be soved with .map -----

  const ImmutableList = [1,2,3,4] 
  // itterates over exsisting, but creates a new list,
  // and doesn't mutate exsisting. Also alot cleaner to read.
  const powerOfTwo = list => list.map(i => i ** 2)

  console.log(powerOfTwo(ImmutableList)) //[1, 4, 9, 16]
  console.log(ImmutableList) //[1, 2, 3, 4]
  ```

<a name="remove-dead-code"></a><a name="2.5"></a>
- #### [2.5](#remove-dead-code) Make it easier for you, remove dead code.

  Why? 
  > Dead code, that is either just unused or commented away makes it hear to read and navigate, if you need it later on cherry-pick it from git.

  ```tsx
  const somelist = items.map(x => {...})
  // var objs = [];
  // Object.keys(p).forEach(key => {
  //   headers.Headers.forEach(h => {
  //     let wordcount = 0;
  //     h.Fields.forEach(element => {
  //       if (element.Title === key) {
  //         if (p[key]) {
  //           element.TextToEdit = p[key];
  //           wordcount += element.TextToEdit.split(" ").length;
  //         } else {
  //           element.TextToEdit = "";
  //         }
  //       }
  //     });
  //     if (wordcount > 50) {
  //       h.Wordcount = wordcount;
  //       //console.log("newheader", h);
  //     }
  //   });
  //   if (key.indexOf("odata.") == -1 && key.toLowerCase() != "id") {
  //     objs.push({
  //       text: p[key],
  //       fieldname: key,
  //       fieldnamedisplay: listItems.items[key]
  //     });
  //   }
  // });
  // var p = items[0];

  // //console.log('Headers2', headers);
  // //console.log('Ny plan', p);
  //this.setState({ plan: p, estimatedWorkload: p.EstimatedWorkload, accordionitems: objs, theheaders: headers, headercoll: headers.Headers }, () => { resolve(); });
  //console.log(headers.Headers);
  resolve(...somelist);
  ```
  #### try

  ```javascript
  const somelist = items.map(x => {...})
  resolve(...somelist);
  ```


<a name="no-promise-wrapping"></a><a name="2.6"></a>
- #### [2.6](#no-promise-wrapping) Don't wrap promises in promises

  Why?
  > Wrapping promsises in other promises causes alot of extra ambiguity and more overhead. 

  ```tsx
  const async fetchThisAsync = () => {...} // Returns a Proise
  const async fetchThisAlsoAsync = () => {...} // Returns a Promise  


  const fetchBothAsync = () => {
    // This has no reject so if this fails Preomise.all will never resolve. 
    // and does a own setState that maybe isn't needed.
    const first = new Promise<void>((resolve, reject) => {
      fetchThisAsync().then(a =>{
        this.setState({a})
        resolve()
      })
    })

    // Handels everything correctly, but also is redundant as this would likely opperate as the promisse given by fetchThisAlsoAsync()
    const second = new Promise<any>((resolve, reject) => {
      fetchThisAlsoAsync().then(a =>{
        resolve(a)
      }).catch(e => reject(e))
    })

    return [first, second]
  }
  
  // Waiting for both to resolve, but also creates alot of ambiguity when actually opperations are executed.
  const wat = new Promise.all(fetchBothAsync()).then([_, second] => 
    if(!!second) { this.setState({second})}
  )
  ```

  #### better way to solve it is

  ```javascript
  const async fetchThisAsync = () => {...} // Returns a Proise
  const async fetchThisAlsoAsync = () => {...} // Returns a Promise  

  // If you need to update as soon as it's resolved and not wait for eachother...
  fetchThisAsync().then(({key, value}) => {
    this.setState({[key]: value})
  }).catch(e => {this.setState({error: e})});

  fetchThisAlsoAsync().then(({key, value}) => {
    this.setState({[key]: value})
  }).catch(e => {this.setState({error: e})})

  // OR if you need both to resolve, before doing update you can for example do.
  Promise.all([fetchThisAsync(), fetchThisAlsoAsync()]).then(resolveResults => {
    const resolvedObj = resolveResults.reduce((acc, {key, value}) => ({...acc, [key]: value}), {})
    this.setState(resolveObj)
  }).catch(e => this.setState({error: e}))
  
  ```

## Objects

<a name="objects--no-new"></a><a name="3.1"></a>
  - #### [3.1](#objects--no-new) Use the literal syntax for object creation. eslint: [`no-new-object`](https://eslint.org/docs/rules/no-new-object.html)

    ```javascript
    // bad
    const item = new Object();

    // good
    const item = {};
    ```

  <a name="es6-computed-properties"></a><a name="3.4"></a>
  - #### [3.2](#es6-computed-properties) Use computed property names when creating objects with dynamic property names.

    Why?
    > They allow you to define all the properties of an object in one place.

    ```javascript

    function getKey(k) {
      return `a key named ${k}`;
    }

    // bad
    const obj = {
      id: 5,
      name: 'San Francisco',
    };
    obj[getKey('enabled')] = true;

    // good
    const obj = {
      id: 5,
      name: 'San Francisco',
      [getKey('enabled')]: true,
    };
    ```

  <a name="es6-object-shorthand"></a><a name="3.5"></a>
  - #### [3.3](#es6-object-shorthand) Use object method shorthand. eslint: [`object-shorthand`](https://eslint.org/docs/rules/object-shorthand.html)

    ```javascript
    // bad
    const atom = {
      value: 1,

      addValue: function (value) {
        return atom.value + value;
      },
    };

    // good
    const atom = {
      value: 1,

      addValue(value) {
        return atom.value + value;
      },
    };
    ```

  <a name="es6-object-concise"></a><a name="3.6"></a>
  - #### [3.4](#es6-object-concise) Use property value shorthand. eslint: [`object-shorthand`](https://eslint.org/docs/rules/object-shorthand.html)

    Why?
    > It is shorter and descriptive.

    ```javascript
    const lukeSkywalker = 'Luke Skywalker';

    // bad
    const obj = {
      lukeSkywalker: lukeSkywalker,
    };

    // good
    const obj = {
      lukeSkywalker,
    };
    ```

  <a name="objects--grouped-shorthand"></a><a name="3.7"></a>
  - #### [3.5](#objects--grouped-shorthand) Group your shorthand properties at the beginning of your object declaration.

    Why?
    > It’s easier to tell which properties are using the shorthand.

    ```javascript
    const anakinSkywalker = 'Anakin Skywalker';
    const lukeSkywalker = 'Luke Skywalker';

    // bad
    const obj = {
      episodeOne: 1,
      twoJediWalkIntoACantina: 2,
      lukeSkywalker,
      episodeThree: 3,
      mayTheFourth: 4,
      anakinSkywalker,
    };

    // good
    const obj = {
      lukeSkywalker,
      anakinSkywalker,
      episodeOne: 1,
      twoJediWalkIntoACantina: 2,
      episodeThree: 3,
      mayTheFourth: 4,
    };
    ```

  <a name="objects--quoted-props"></a><a name="3.8"></a>
  - #### [3.6](#objects--quoted-props) Only quote properties that are invalid identifiers. eslint: [`quote-props`](https://eslint.org/docs/rules/quote-props.html)

     Why? 
     > In general we consider it subjectively easier to read. It improves syntax highlighting, and is also more easily optimized by many JS engines.

    ```javascript
    // bad
    const bad = {
      'foo': 3,
      'bar': 4,
      'data-blah': 5,
    };

    // good
    const good = {
      foo: 3,
      bar: 4,
      'data-blah': 5,
    };
    ```

  <a name="objects--prototype-builtins"></a>
  - #### [3.7](#objects--prototype-builtins) Do not call `Object.prototype` methods directly, such as `hasOwnProperty`, `propertyIsEnumerable`, and `isPrototypeOf`. eslint: [`no-prototype-builtins`](https://eslint.org/docs/rules/no-prototype-builtins)

    Why?
    > These methods may be shadowed by properties on the object in question - consider `{ hasOwnProperty: false }` - or, the object may be a null object (`Object.create(null)`).

    ```javascript
    // bad
    console.log(object.hasOwnProperty(key));

    // good
    console.log(Object.prototype.hasOwnProperty.call(object, key));

    // best
    const has = Object.prototype.hasOwnProperty; // cache the lookup once, in module scope.
    console.log(has.call(object, key));
    /* or */
    import has from 'has'; // https://www.npmjs.com/package/has
    console.log(has(object, key));
    ```

  <a name="objects--rest-spread"></a>
  - #### [3.8](#objects--rest-spread) Prefer the object spread operator over [`Object.assign`](https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Global_Objects/Object/assign) to shallow-copy objects. Use the object rest operator to get a new object with certain properties omitted.

    ```javascript
    // very bad
    const original = { a: 1, b: 2 };
    const copy = Object.assign(original, { c: 3 }); // this mutates `original` ಠ_ಠ
    delete copy.a; // so does this

    // bad
    const original = { a: 1, b: 2 };
    const copy = Object.assign({}, original, { c: 3 }); // copy => { a: 1, b: 2, c: 3 }

    // good
    const original = { a: 1, b: 2 };
    const copy = { ...original, c: 3 }; // copy => { a: 1, b: 2, c: 3 }

    const { a, ...noA } = copy; // noA => { b: 2, c: 3 }
    ```

## Arrays

  <a name="arrays--literals"></a><a name="4.1"></a>
  - #### [4.1](#arrays--literals) Use the literal syntax for array creation.
    ```javascript
    // bad
    const items = new Array();

    // good
    const items = [];
    ```

  <a name="arrays--push"></a><a name="4.2"></a>
  - #### [4.2](#arrays--push) Use [Array#push](https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Global_Objects/Array/push) instead of direct assignment to add items to an array.

    ```javascript
    const someStack = [];

    // bad
    someStack[someStack.length] = 'abracadabra';

    // good
    someStack.push('abracadabra');
    ```

  <a name="es6-array-spreads"></a><a name="4.3"></a>
  - #### [4.3](#es6-array-spreads) Use array spreads `...` to copy arrays.

    ```javascript
    // bad
    const len = items.length;
    const itemsCopy = [];
    let i;

    for (i = 0; i < len; i += 1) {
      itemsCopy[i] = items[i];
    }

    // good
    const itemsCopy = [...items];
    ```


  <a name="arrays--callback-return"></a><a name="4.4"></a>
  - #### [4.4](#arrays--callback-return) Use return statements in array method callbacks. It’s ok to omit the return if the function body consists of a single statement returning an expression without side effects, following [8.2](#arrows--implicit-return).

    ```javascript
    // good
    [1, 2, 3].map((x) => {
      const y = x + 1;
      return x * y;
    });

    // good
    [1, 2, 3].map((x) => x + 1);

    // bad - no returned value means `acc` becomes undefined after the first iteration
    [[0, 1], [2, 3], [4, 5]].reduce((acc, item, index) => {
      const flatten = acc.concat(item);
    });

    // good
    [[0, 1], [2, 3], [4, 5]].reduce((acc, item, index) => {
      const flatten = acc.concat(item);
      return flatten;
    });

    // bad
    inbox.filter((msg) => {
      const { subject, author } = msg;
      if (subject === 'Mockingbird') {
        return author === 'Harper Lee';
      } else {
        return false;
      }
    });

    // good
    inbox.filter((msg) => {
      const { subject, author } = msg;
      if (subject === 'Mockingbird') {
        return author === 'Harper Lee';
      }

      return false;
    });
    ```
